"""
Shared values

Copyright 2015, Outernet Inc.
Some rights reserved.

This software is free software licensed under the terms of GPLv3. See COPYING
file that comes with the source code, or http://www.gnu.org/licenses/gpl.txt.
"""

import re

import validators as v


CONTENT_ID_RE = re.compile(r'^[0-9a-f]{32}$', re.I)
PLACEHOLDER_RE = re.compile(r'^\$[A-Z]+$')
LOCALE_RE = re.compile(r'^[a-z]{2}([_-][a-zA-Z]+)?$', re.I)
COMMASEP_RE = re.compile(r'^[\w ]+(?:, ?[\w ]+)*$', re.U)
RELPATH_RE = re.compile(r'^[^/]+(/[^/]+)*$')
TS_FMT = '%Y-%m-%d %H:%M:%S UTC'
DATE_FMT = '%Y-%m-%d'
LICENSES = ('CC-BY', 'CC-BY-ND', 'CC-BY-NC', 'CC-BY-ND-NC', 'CC-BY-SA',
            'CC-BY-NC-SA', 'GFDL', 'OPL', 'OCL', 'ADL', 'FAL', 'PD', 'OF',
            'ARL', 'ON')
LICENSE_NAMES = (
    'Creative Commons Attribution',
    'Creative Commons Attribution-NoDerivs',
    'Creative Commons Attribution-NonCommercial',
    'Creative Commons Attribution-NonCommercial-NoDerivs',
    'Creative Commons Attribution-ShareAlike',
    'Creative Commons Attribution-NonCommercial-ShareAlike',
    'GNU Free Documentation License',
    'Open Publication License',
    'Open Content License',
    'Against DRM License',
    'Free Art License',
    'Public Domain',
    'Other free license',
    'All rights reserved',
    'Other non-free license',
)
LICENSE_PAIRS = dict(zip(LICENSES, LICENSE_NAMES))

REQUIRED = (
    'title',
    'url',
    'timestamp',
    'broadcast',
    'license',
)

OPTIONAL = (
    'archive',
    'images',
    'index',
    'is_partner',
    'is_sponsored',
    'keep_formatting',
    'keywords',
    'language',
    'multipage',
    'partner',
    'publisher',
    'replaces',
)

KEYS = REQUIRED + OPTIONAL

DEFAULTS = {
    'archive': 'core',
    'images': 0,
    'index': 'index.html',
    'is_partner': False,
    'is_sponsored': False,
    'keep_formatting': False,
    'keywords': '',
    'language': '',
    'multipage': False,
    'partner': '',
    'publisher': '',
}

SPECS = {
    'title': [v.required, v.nonempty],
    'url': [v.required, v.nonempty, v.url],
    'timestamp': [v.required, v.nonempty, v.timestamp(TS_FMT)],
    'broadcast': [v.required, v.nonempty,
                  v.OR(v.timestamp(DATE_FMT), v.match(PLACEHOLDER_RE))],
    'license': [v.required, v.isin(LICENSES)],
    'images': [v.optional(), v.istype(int), v.gte(0)],
    'language': [v.optional(''), v.nonempty, v.match(LOCALE_RE)],
    'multipage': [v.optional(), v.istype(bool)],
    'index': [v.optional(''), v.match(RELPATH_RE)],
    'keywords': [v.optional(''), v.nonempty, v.match(COMMASEP_RE)],
    'archive': [v.optional(''), v.nonempty],
    'partner': [v.optional(''), v.nonempty],
    'publisher': [v.optional(''), v.nonempty],
    'is_partner': [v.optional(), v.istype(bool)],
    'is_sponsored': [v.optional(), v.istype(bool)],
    'keep_formatting': [v.optional(), v.istype(bool)],
    'replaces': [v.optional(''), v.match(CONTENT_ID_RE)],
}



"""
Functions for working with paths

Copyright 2015, Outernet Inc.
Some rights reserved.

This software is free software licensed under the terms of GPLv3. See COPYING
file that comes with the source code, or http://www.gnu.org/licenses/gpl.txt.
"""

import os


def fnwalk(path, fn):
    """
    Walk directory tree top-down until directories of desired length are found

    This generator function takes a ``path`` from which to begin the traversal,
    and a ``fn`` object that selects the paths to be returned. It calls
    ``os.listdir()`` recursively until either a full path is flagged by ``fn``
    function as valid (by returning a truthy value) or ``os.listdir()`` fails
    with ``OSError``.

    This function has been added specifically to deal with large and deep
    directory trees, and it's tehrefore not avisable to convert the return
    values to a lists and similar memory-intensive objects.
    """
    if fn(path):
        yield path

    try:
        names = os.listdir(path)
    except OSError:
        return

    for name in names:
        for child in fnwalk(os.path.join(path, name), fn):
            yield child


def extwalk(path, exts=['.jpg', '.jpeg', '.png', '.gif', '.svg']):
    """
    Walk directory tree top-down until files with given extensions are matched.

    Default set of extensions match common image formats including SVG. The
    extension list can be customized by providing a non-generator iterable
    (e.g., list, tuple, dict) where each memeber is an extension including the
    dot.
    """
    matchfn = lambda p: os.path.isfile(p) and os.path.splitext(p)[1] in exts
    for p in fnwalk(matchfn):
        yield p


def extcount(path, exts=['.jpg', '.jpeg', '.png', '.gif', '.svg']):
    """
    Return a count of all files matching given extensions in a certain path

    Default set of extensions match common image formats including SVG. The
    extension list can be customized by providing a non-generator iterable
    (e.g., list, tuple, dict) where each memeber is an extension including the
    dot.
    """
    if os.path.isfile(path):
        return os.path.splitext(path)[1] in exts
    sub = os.listdir(path)
    total_found = 0
    for s in sub:
        total_found += extcount(os.path.join(path, s), exts)
    return total_found



"""
Functions for working with program arguments

Copyright 2015, Outernet Inc.
Some rights reserved.

This software is free software licensed under the terms of GPLv3. See COPYING
file that comes with the source code, or http://www.gnu.org/licenses/gpl.txt.
"""

import sys
import platform
import argparse

from . import __version__, __copyright__

VERSION = "%(prog)s {} / Python {} {}".format(
    __version__,
    '.'.join([str(s) for s in sys.version_info[:3]]),
    ' '.join(platform.architecture()))


def getparser(desc, usage=None):
    parser = argparse.ArgumentParser(
        description=desc,
        usage=usage,
        epilog=__copyright__)
    parser.add_argument('--version', '-V', action='version', version=VERSION)
    return parser



"""
Functions for handling signals

Copyright 2015, Outernet Inc.
Some rights reserved.

This software is free software licensed under the terms of GPLv3. See COPYING
file that comes with the source code, or http://www.gnu.org/licenses/gpl.txt.
"""

from __future__ import print_function

import sys
import signal


def oninterrupt(signum, frame):
    print('\nProgram was terminated', file=sys.stderr)
    sys.exit(1)


signal.signal(signal.SIGINT, oninterrupt)
signal.signal(signal.SIGTERM, oninterrupt)



"""
Validator to use a library

Copyright 2015, Outernet Inc.
Some rights reserved.

This software is free software licensed under the terms of GPLv3. See COPYING
file that comes with the source code, or http://www.gnu.org/licenses/gpl.txt.
"""

import validators

from . import values


VALIDATOR = validators.spec_validator(
    values.SPECS, key=lambda k: lambda obj: obj.get(k))


def validate(data, broadcast=False):
    """ Validates data

    When ``broadcast`` flag is ``True``, then the placeholder value for
    ``broadcast`` is not allowed.
    """
    res = VALIDATOR(data)
    if res:
        return res
    # Strict checking for broadcast
    if broadcast:
        if data['broadcast'] == '$BROADCAST':
            # Not allowing placeholders in strict mode
            raise ValueError('broadcast date cannot be a placeholder',
                             'broadcast_strict')
    # Additional validation that cannot be done using the specs
    if 'publisher' not in data or 'partner' not in data:
        return {}
    if data['publisher'] == data['partner']:
        return {}
    return {
        'publisher': ValueError('must match partner', 'match_other'),
        'partner': ValueError('must match publisher', 'match_other')
    }



#!/usr/bin/env python

"""
Validate metadata

Copyright 2015, Outernet Inc.
Some rights reserved.

This software is free software licensed under the terms of GPLv3. See COPYING
file that comes with the source code, or http://www.gnu.org/licenses/gpl.txt.
"""

import json

import conz

from . import validator

cn = conz.Console()

try:
    FILE_ERRORS = (IOError, OSError, FileNotFoundError)
except NameError:
    FILE_ERRORS = (IOError, OSError,)


def load(path):
    """ Load JSON data from file """
    with open(path, 'r') as f:
        data = json.load(f)
    return data


def validate_path(path):
    path = path.strip()
    try:
        data = load(path)
    except FILE_ERRORS:
        cn.pverr(path, 'file not found')
        raise RuntimeError()
    except ValueError:
        cn.pverr(path, 'invalid JSON format')
        raise RuntimeError()
    errors = validator.validate(data)
    if errors:
        cn.pstd(cn.color.red('{} ERR'.format(path)))
        for key, err in sorted(errors.items(), key=lambda x: x[0]):
            err, _ = err.args
            cn.pverb('{}: {}'.format(key, cn.color.red(err)))
        return 1
    cn.pstd(cn.color.green('{} OK'.format(path)))
    return 0


def main():
    from .argutil import getparser

    parser = getparser('Validate metadata file',
                       usage='\n    %(prog)s [-h] [-V] PATH\n    '
                       'PATH | %(prog)s [-h] [-V]')
    parser.add_argument('paths', metavar='PATH', help='optional path to '
                        'metadata file (defaults to info.json in current '
                        'directory, ignored if used in a pipe)',
                        default=['./info.json'], nargs='*')
    args = parser.parse_args()

    if cn.interm:
        cn.verbose = True
        src = args.paths
    else:
        src = cn.readpipe()

    for p in src:
        try:
            validate_path(p)
        except RuntimeError:
            cn.pstd(cn.color.red('{} ERR'.format(p)))


if __name__ == '__main__':
    main()



#!/usr/bin/env python

"""
Validate metadata

Copyright 2015, Outernet Inc.
Some rights reserved.

This software is free software licensed under the terms of GPLv3. See COPYING
file that comes with the source code, or http://www.gnu.org/licenses/gpl.txt.
"""

import json
import hashlib
import datetime

import sys
import validators
import conz

from . import values


PY3 = sys.version_info >= (3, 0, 0)
if PY3:
    FILE_OPTS = {'encoding': 'utf8'}
else:
    FILE_OPTS = {}

JSON_OPTS = {'indent': 4, 'sort_keys': True}

cn = conz.Console()


def valwrap(fn):
    """ Wrap validator function so it returns False instead of raising """
    def wrapper(x):
        try:
            fn(x)
            return True
        except ValueError:
            return False
    return wrapper


def generate_template(**kwargs):
    """ Generate metadta template dict from given kwargs

    Keyword arguments that are passed will be added to the generated template
    only if they are named as valid keys. In all other cases, they are simply
    disregarded.

    For all required key, an empty string will be used as key value. This
    results in invalid metadata, but since the purpose of this function is to
    generate a template, no validation is performed. It is expected that the
    caller will perform any necessary validation.

    For optional keys, default values are provided as per the specification_.

    .. _specification: http://j.mp/outernet-metadata
    """
    data = {}
    for k in values.REQUIRED:
        data[k] = kwargs.get(k, '')
    for k, v in values.DEFAULTS.items():
        data[k] = kwargs.get(k, v)
    data['broadcast'] = data['broadcast'] or '$BROADCAST'
    return data


def ask_title():
    """ Get content title from user input """
    return cn.rvpl('title:', validator=valwrap(validators.nonempty),
                   intro="""
                   Title of the content will appear in the Outernet's content
                   lists.
                   """)


def ask_url():
    """ Gets URL from user input """
    cn.pstd()
    return cn.rvpl('URL:', validator=valwrap(validators.url),
                   intro="""
                   URL of the content package is required in order to generate
                   the package template. This URL may be a valid web URL if the
                   content comes from the web, or an URL in the following form:
                   outernet://name.com/path where "name" is any name of your
                   choosing that contains only alphanumeric characetrs (a-z,
                   0-9) and "path" is an arbitrary path specific to your
                   content (e.g., /my-awesome-content), and it needs to contain
                   only alphanumeric characters, dashes and underscores.
                   """)


def ask_keywords():
    """ Gets comma-separate keywords from user input """
    cn.pstd()
    validator = validators.match(values.COMMASEP_RE)
    ret = cn.rvpl('keywords []:', validator=valwrap(validator),
                  intro="""
                  Keywords are used when searching and classifying content.
                  Keywords cannot contain commas, and multiple keywords are
                  supplied separated by commas. For example:
                  medical,medicine,farming,plant,biology
                  """, strict=False, default='')
    kws = [s.strip() for s in ret.split(',')]
    return ','.join([s for s in kws if s])


def ask_language():
    """ Get content locale """
    print()
    validator = validators.match(values.LOCALE_RE)
    return cn.rvpl('language []:', validator=valwrap(validator),
                   intro="""
                   Content language is specified standard ISO 639-1 locale
                   codes. For example: pt_BR. You will find a list of locale
                   codes at:

                   http://loc.gov/standards/iso639-2/php/code_list.php

                   Language is optional, but it allows users to search content
                   in specific language, so it is recommended to set it.
                   """, strict=False, default='')


def ask_publisher():
    """ Get content attribution """
    print()
    return cn.rvpl('publisher []:',
                   intro="""
                   Enter the name of a person or an entity that published this
                   content.  You may leave this blank if not applicable.
                   """, strict=False, default='')


def ask_license():
    print()
    return cn.menu(list(zip(values.LICENSES, values.LICENSE_NAMES)),
                   'license:')


def ask_timestamp():
    print()
    validator = validators.timestamp(values.TS_FMT[:-4])
    ret = cn.rvpl('timestamp [current time]:', validator=valwrap(validator),
                  intro="""
                  Timestamp usually refers to the time when content was
                  packaged. The timestamp should be in UTC and in the following
                  format: YYYY-MM-DD HH:MM:SS. You may leave this blank to use
                  the current time.
                  """, strict=False)
    if not ret:
        return datetime.datetime.utcnow().strftime(values.TS_FMT)
    else:
        return ret + ' UTC'


def ask_archive():
    print()
    return cn.rvpl('archive [community]',
                   intro="""
                   Archives are sections of the Outernet library. If you are
                   not sure which archive this content belongs to, contact
                   Outernet staff or leave blank.
                   """, strict=False, default='community')


def ask_index():
    print()
    validator = validators.match(values.RELPATH_RE)
    return cn.rvpl('index [index.html]:', validator=valwrap(validator),
                   intro="""
                   Index points to the file that should seve as main page of
                   the content.  It should be a relative path relative to the
                   top level path of the content package. If left blank,
                   index.html is assumed.
                   """, strict=False, default='index.html')


def ask_multipage():
    print()
    return cn.yesno('multipage?', default=False,
                    intro="""
                    If you content consists of multiple interlinked pages,
                    select "yes".  Multipage content must use relative paths to
                    reference pages and assets within the package. Using
                    absolute paths will have unintended side-effects and
                    usually result in "Page not found" errors.
                    """)


def guide():
    data = {}
    data['title'] = ask_title()
    data['publisher'] = data['partner'] = ask_publisher()
    data['url'] = ask_url()
    data['keywords'] = ask_keywords()
    data['archive'] = ask_archive()
    data['language'] = ask_language()
    data['license'] = ask_license()
    data['timestamp'] = ask_timestamp()
    data['index'] = ask_index()
    data['multipage'] = ask_multipage()
    data['is_sponsored'] = False
    data['is_partner'] = False
    return data


def md5(s):
    """ Returns MD5 hexdigest for given string """
    h = hashlib.md5()
    h.update(s.encode('utf8'))
    return h.hexdigest()


def main():
    import os
    import argparse

    from .argutil import getparser

    parser = getparser('Generate metadata template')
    output = parser.add_mutually_exclusive_group()
    output.add_argument('--out', '-o', metavar='PATH', default=None,
                        type=argparse.FileType('w', **FILE_OPTS),
                        help='write just the metadata to a file')
    output.add_argument('--package', '-p', action='store_true',
                        help='create a package template')
    parser.add_argument('--guided', '-g', action='store_true',
                        help='guided metadata creation')
    args = parser.parse_args()

    if args.guided:
        meta = generate_template(**guide())
    else:
        meta = generate_template()

    if args.out:
        json.dump(meta, args.out, **JSON_OPTS)
    elif args.package:
        url = meta['url'] or ask_url()
        meta['url'] = url
        id = md5(url)
        os.makedirs(id)
        with open(os.path.join(id, 'info.json'), 'w', **FILE_OPTS) as f:
            json.dump(meta, f, **JSON_OPTS)
    else:
        print(json.dumps(meta, **JSON_OPTS))


if __name__ == '__main__':
    main()



#!/usr/bin/env python

import os
import sys
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand

SCRIPTDIR = os.path.dirname(__file__) or '.'
PY3 = sys.version_info >= (3, 0, 0)

from outernet_metadata import __version__


def read(fname):
    """ Return content of specified file """
    path = os.path.join(SCRIPTDIR, fname)
    if PY3:
        f = open(path, 'r', encoding='utf8')
    else:
        f = open(path, 'r')
    content = f.read()
    f.close()
    return content


class PyTest(TestCommand):
    user_options = [('pytest-args=', 'a', 'Arguments for py.test')]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        self.pytest_args = []

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run(self):
        import pytest
        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


setup(
    name='outernet-metadata',
    version=__version__,
    author='Outernet Inc',
    author_email='apps@outernet.is',
    description='Library for working with Outernet metadata',
    license='GPLv3',
    keywords='json, validation, templates, metadata, outernet',
    url='https://github.com/Outernet-Project/bottle-fdsend',
    packages=find_packages(),
    long_description=read('README.rst'),
    install_requires=[
        'chainable-validators>=0.5',
    ],
    extras_require={
        'command line tools':  ['conz>=0.3'],
    },
    entry_points={
        'console_scripts': [
            'metacheck = outernet_metadata.validate:main',
            'metagen = outernet_metadata.template:main',
            'imgcount = outernet_metadata.imgcount:main',
        ],
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 2.7',
    ],
    cmdclass={
        'test': PyTest,
    },
)



#!/usr/bin/env python

"""
Count images in folders

Copyright 2015, Outernet Inc.
Some rights reserved.

This software is free software licensed under the terms of GPLv3. See COPYING
file that comes with the source code, or http://www.gnu.org/licenses/gpl.txt.
"""

from __future__ import print_function

import os
import sys
import json

from . import flowutil
from .pathutil import extcount

PY3 = sys.version_info >= (3, 0, 0)
if PY3:
    FILE_OPTS = {'encoding': 'utf8'}
    FILE_ERRORS = (OSError, FileNotFoundError)
else:
    FILE_OPTS = {}
    FILE_ERRORS = (OSError)


def main():
    from .argutil import getparser

    parser = getparser('Calculate the number of images in a folder')
    parser.add_argument('path', metavar='PATH', nargs='?', default='.',
                        help='path to search for images (default is to search '
                        'inside curent directory)')
    parser.add_argument('--update', '-u', action='store_true',
                        help='update the info.json found at PATH/info.json')
    args = parser.parse_args()

    count = extcount(args.path)
    print('images found: {}'.format(count))

    if args.update:
        path = os.path.join(args.path, 'info.json')
        try:
            with open(path, 'r', **FILE_OPTS) as f:
                data = json.load(f)
            data['images'] = count
            with open(path, 'w', **FILE_OPTS) as f:
                json.dump(data, f, indent=4, sort_keys=True)
        except FILE_ERRORS:
            print('{}: read/write error, cannot update metadata'.format(path),
                  file=sys.stderr)
            sys.exit(1)
        except ValueError:
            print('{}: metadta could not be decoded, check syntax',
                  file=sys.stderr)
            sys.exit(1)
        print('updated metadata: {}'.format(path))



__version__ = '0.5.post3'
__author__ = 'Outernet Inc'
__copyright__ = """
Copyright 2015, Outernet Inc.
Some rights reserved.

This software is free software licensed under the terms
of GPLv3. See COPYING file that comes with the source code,
or http://www.gnu.org/licenses/gpl.txt.
"""
